import random
import httpx
import asyncio
from httpx import ConnectError, ConnectTimeout, ProxyError, ReadTimeout, ReadError, UnsupportedProtocol
from curl_cffi import requests
from scan.response import Response
from scan.common import logger
from httpx._config import SSLConfig, DEFAULT_CIPHERS


class Downloader(object):
    client = None
    ssl_context = None

    def create_ssl_context(self, verify):
        ssl_config = SSLConfig(verify=verify)
        ssl_context = ssl_config.load_ssl_context()
        ssl_context.set_ciphers(DEFAULT_CIPHERS + 'HIGH:!DH')
        self.ssl_context = ssl_context

    @staticmethod
    async def gen_headers():
        kit = f'{random.randint(490, 540)}.{random.randint(10, 90)}'
        return {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;'
                      'q=0.8,application/signed-exchange;v=b3;q=0.9',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'en;q=0.9',
            'User-Agent': f'Mozilla/5.0 (Macintosh; Intel Mac OS X {random.randint(6, 15)}_{random.randint(2, 20)}'
                          f'_{random.randint(1, 9)}) AppleWebKit/{kit} (KHTML, like Gecko) '
                          f'Chrome/{random.randint(95, 98)}.0.{random.randint(1000, 5000)}.{random.randint(100, 200)}'
                          f' Safari/{kit}'
        }

    async def close(self):
        try:
            await self.client.aclose()
        except:
            pass

    @staticmethod
    async def log_request(request):
        pass

    @staticmethod
    async def log_response(response):
        """
        日志钩子
        """
        request = response.request
        if response.status_code not in [200, 301, 302]:
            logger.error(f'{response.status_code}  {request.url}')
        else:
            logger.info(f'{response.status_code}  {request.url}')

    async def tls_request(self, method, url, data=None, json=None, proxies=None):
        response = Response()
        try:
            if proxies:
                proxy = {'https': proxies.get('https://'), 'http': proxies.get('http://')}
            else:
                proxy = None
            resp = requests.request(method=method, url=url, data=data, json=json,  impersonate="chrome101",
                                    proxies=proxy)
            await self.log_response(resp)
            response.response = resp
            response.ok = True
        except Exception as e:
            logger.error(f'tls request error: {e}')
        return response

    async def request(self, url, params=None, headers=None, cookies=None, auth=None, proxies=None, allow_redirects=True,
                      verify=True, http2=False,  content=None, data=None, files=None, json=None, stream=False,
                      timeout=30, cycle=3, tls=False, session=True):
        if not self.ssl_context:
            self.create_ssl_context(verify)
        if data or json or content:
            method = 'POST'
        else:
            method = 'GET'
        if not headers:
            headers = await self.gen_headers()
        if tls:
            return await self.tls_request(method=method, url=url, data=data, json=json, proxies=proxies)
        response = Response()
        response.request_url = url
        for _ in range(cycle):
            try:
                if session:
                    if not self.client:
                        async with asyncio.Lock():
                            self.client = httpx.AsyncClient(proxies=proxies, verify=verify, http2=http2,
                                                            event_hooks={'response': [self.log_response]})
                            try:
                                list(self.client._mounts.values())[0]._pool._ssl_context = self.ssl_context
                            except:
                                pass
                    else:
                        self.client.proxies = proxies
                    resp = await self.client.request(method=method, url=url, content=content, data=data, files=files,
                                                     json=json, params=params, headers=headers, cookies=cookies,
                                                     auth=auth, follow_redirects=allow_redirects, timeout=timeout)
                    response.response = resp
                    response.ok = True
                elif stream:
                    client = httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]},
                                               verify=verify, http2=http2, follow_redirects=allow_redirects)
                    try:
                        list(client._mounts.values())[0]._pool._ssl_context = self.ssl_context
                    except:
                        pass
                    request = client.build_request(method=method, url=url, headers=headers, cookies=cookies,
                                                   content=content, data=data, files=files, json=json, timeout=timeout)
                    resp = await client.send(request, stream=True)
                    response.response = resp
                    response.ok = True
                    response.client = client  # need aclose
                else:
                    async with httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]},
                                                 verify=verify, http2=http2) as client:
                        try:
                            list(client._mounts.values())[0]._pool._ssl_context = self.ssl_context
                        except:
                            pass
                        resp = await client.request(
                            method=method, url=url, content=content, data=data, files=files, json=json, params=params,
                            headers=headers, cookies=cookies, auth=auth, follow_redirects=allow_redirects,
                            timeout=timeout
                        )
                        response.response = resp
                        response.ok = True
                if resp.status_code == 403:
                    return await self.tls_request(method=method, url=url, data=data, json=json, proxies=proxies)
                return response
            except ConnectError as e:
                response.message = 'ConnectError'
                logger.error(f'Failed to request {url}  ConnectError:{e}')
            except ConnectTimeout as e:
                response.message = 'ConnectTimeout'
                logger.error(f'Failed to request {url}  ConnectTimeout:{e}')
            except ProxyError as e:
                response.message = 'ProxyError'
                logger.error(f'Failed to request {url}  ProxyError:{e}')
            except ReadTimeout as e:
                response.message = 'ReadTimeout'
                logger.error(f'Failed to request {url}  ReadTimeout:{e}')
            except ReadError as e:
                response.message = 'ReadError'
                logger.error(f'Failed to request {url}  ReadError:{e}')
            except UnsupportedProtocol as e:
                response.message = 'UnsupportedProtocol'
                logger.error(f'Failed to request {url}  UnsupportedProtocol:{e}')
            except Exception as e:
                response.message = e
                logger.error(f'Failed to request {url}  {e}')
        return response







import json
import aioredis
from scan.common import logger


class Redis:
    def __init__(self, **kwargs):
        self.host = kwargs.get('host') or 'localhost'
        self.port = kwargs.get('port') or 6379
        self.password = kwargs.get('password')
        self.db = kwargs.get('db') or 1
        self.pool_size = kwargs.get('pool_size') or 10
        self.user = kwargs.get('user')
        self.redis_conn = None
        self.group_name = None

    async def get_redis(self, pool_size=None):
        if pool_size and isinstance(pool_size, int):
            self.pool_size = pool_size
        pool = aioredis.ConnectionPool.from_url(f"redis://{self.host}", port=int(self.port), username=self.user, password=self.password,
                                                db=int(self.db), max_connections=int(self.pool_size))
        redis = aioredis.Redis(connection_pool=pool)
        self.redis_conn = redis
        return redis

    async def publish(self, data, queue_name, priority=None):
        if not self.redis_conn:
            await self.get_redis()
        try:
            data = json.dumps(data)
            res = await self.redis_conn.lpush(queue_name, data)
            if not res:
                logger.error(f'redis queue publish message error: {queue_name} {data}')
        except Exception as e:
            logger.error(f'publish process error: {e}')

    async def consume(self, queue_name, count=1):
        if not self.redis_conn:
            await self.get_redis()
        try:
            message = await self.redis_conn.rpop(queue_name)
            if message:
                message = json.loads(message.decode())
            return message
        except Exception as e:
            logger.error(f'consume process error: {e}')

    async def ack(self, queue_name, message):
        try:
            await self.redis_conn.srem(f'temp_{queue_name}', json.dumps(message))
        except Exception as e:
            logger.error(f'ack message error:{e}')

    async def resend(self, data, queue_name):
        await self.publish(data, queue_name)


__all__ = Redis



import io
import aioboto3
import botocore.exceptions
from scan.common import logger


class AioCFR2:
    def __init__(self, **kwargs):
        self.account_id = kwargs.get('account_id')
        self.aws_access_key_id = kwargs.get('aws_access_key_id')
        self.aws_secret_access_key = kwargs.get('aws_secret_access_key')

    async def upload(self, bucket_name, resp, file_name):
        """
        :param bucket_name:
        :param resp:
        :param file_name:
        :return:
        小文件上传
        """
        try:
            async with aioboto3.Session().client('s3',
                                                 endpoint_url=f'https://{self.account_id}.r2.cloudflarestorage.com',
                                                 aws_access_key_id=self.aws_access_key_id,
                                                 aws_secret_access_key=self.aws_secret_access_key) as client:
                content = resp.content()
                byte_stream_file_obj = io.BytesIO(content)
                try:
                    # 上传字节流文件对象到S3存储桶中
                    res = await client.upload_fileobj(byte_stream_file_obj, bucket_name, file_name)
                    logger.info(f'Uploaded {file_name} to {bucket_name}')
                    return True
                except botocore.exceptions.ClientError as e:
                    logger.error(f'Error uploading {file_name}: {e}')
                return False
        except Exception as e:
            logger.error(f'Upload process error: {e}')

    async def upload_bytes(self, bucket_name, file_bytes, file_name):
        """
        :param bucket_name:
        :param file_bytes:
        :param file_name:
        :return:
        小文件上传
        """
        try:
            byte_stream_file_obj = io.BytesIO(file_bytes)
            async with aioboto3.Session().client('s3',
                                                 endpoint_url=f'https://{self.account_id}.r2.cloudflarestorage.com',
                                                 aws_access_key_id=self.aws_access_key_id,
                                                 aws_secret_access_key=self.aws_secret_access_key) as client:
                # 上传字节流文件对象到S3存储桶中
                await client.upload_fileobj(byte_stream_file_obj, bucket_name, file_name)
                logger.info(f'Uploaded {file_name} to {bucket_name}')
                return True
        except botocore.exceptions.ClientError as e:
            logger.error(f'Error uploading {file_name}: {e}')
        return False

    async def upload_large(self, bucket_name, resp, file_name):
        """
        :param bucket_name:  桶名
        :param resp: 流式请求的响应
        :param file_name: 保存的文件名
        :return: 上传成功或失败
        用于处理大文件上传，不过小文件也可以上传
        """
        try:
            async with aioboto3.Session().client('s3',
                                                 endpoint_url=f'https://{self.account_id}.r2.cloudflarestorage.com',
                                                 aws_access_key_id=self.aws_access_key_id,
                                                 aws_secret_access_key=self.aws_secret_access_key) as client:
                # 逐块上传流数据到S3存储桶中
                upload_id = (await client.create_multipart_upload(Bucket=bucket_name, Key=file_name)).get('UploadId')
                part_number = 1
                uploaded_parts = []
                part_size = 5 * 1024 * 1024  # 每块5MB，小了要报错

                async for chunk in resp.response.aiter_bytes(chunk_size=part_size):
                    if chunk:
                        part = await client.upload_part(
                            Body=chunk,
                            Bucket=bucket_name,
                            Key=file_name,
                            PartNumber=part_number,
                            UploadId=upload_id
                        )
                        uploaded_parts.append({'PartNumber': part_number, 'ETag': part['ETag']})
                        part_number += 1

                await client.complete_multipart_upload(
                    Bucket=bucket_name,
                    Key=file_name,
                    UploadId=upload_id,
                    MultipartUpload={'Parts': uploaded_parts}
                )
                logger.info(f'Uploaded {file_name} from stream to {bucket_name}')
                return True
        except Exception as e:
            logger.error(f'Error uploading {file_name}: {e}')
        return False



import json
import time
import aiomysql
from scan import crawl
from hashlib import md5
from scan.common import logger


class Monitor:
    def __init__(self):
        self.__logs = {}

    def _need_send(self, send_data):
        data_md5 = md5(json.dumps(send_data).encode()).hexdigest()
        now = int(time.time())
        for k, v in self.__logs.items():
            if v + 60 * 30 < now:
                self.__logs.pop(k)
        if self.__logs.get(data_md5):
            return False
        else:
            self.__logs.update({data_md5: int(time.time())})
            return True

    async def send_warn_fs(self, web_hook_url, product_name, message, at='', uid=''):
        post_data = {
            "msg_type": "post",
            "content": {
                "post": {
                    "zh_cn": {
                        "title": "爬虫项目异常推送",
                        "content": [
                            [{"tag": "text", "text": f"项目:【{product_name}】"}],
                            [{"tag": "text", "text": f"描述: {message}"}],
                            [{"tag": "text", "text": f"项目负责人:@{at}"}]
                        ]
                    }
                }
            }
        }
        if not self._need_send(post_data):
            return
        try:
            resp = await crawl.fetch(web_hook_url, json=post_data)
            data = resp.json()
            logger.info(f'告警返回:{data}')
        except Exception as e:
            logger.error(f'发送告警数据异常:{e}')

    @staticmethod
    async def async_check_mysql_update_time(conn, db, table, host=None, port=3306, user=None, password=None):
        try:
            if not conn:
                if not host and not port:
                    logger.error('需要数据库连接信息')
                    return
                conn = await aiomysql.connect(host=host, port=port, user=user, password=password)
            async with conn.cursor() as cur:
                find = f"SELECT UPDATE_TIME FROM  information_schema.tables WHERE  TABLE_SCHEMA = '{db}'" \
                       f" AND TABLE_NAME = '{table}'"
                await cur.execute(find)
                db_data = await cur.fetchone()
                timestamp = int(db_data[0].timestamp())
                if int(time.time()) - timestamp > 60 * 30:
                    return False
                return True
        except Exception as e:
            logger.error(f'检查表更新时间异常:{e}')
        finally:
            try:
                conn.close()
            except Exception as e:
                logger.error(f'关闭数据库连接异常:{e}')

    def check_mysql_update_time(self, conn, db, table):
        pass


monitor = Monitor()
__all__ = monitor



import logging


class Logger(object):
    def __init__(self, name=None):
        scan_logger = logging.getLogger(name)
        scan_logger.setLevel(logging.DEBUG)

        # create console handler and set level to debug
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)

        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        ch.setFormatter(formatter)
        scan_logger.addHandler(ch)

        self.logger = scan_logger


def logger(name):
    return Logger(name).logger



from scan.database.rabbitmq import RabbitMQ
from scan.database.redis import Redis
from scan.database.mongodb import MongoDB
from scan.database.oss import OSS
from scan.database.pg import Postgresql
from scan.database.kafka import Kafka
from scan.database.r2 import CFR2
from scan.database.aior2 import AioCFR2
